@@ -51,14 +51,23 @@
 GST_DEBUG_CATEGORY_STATIC (gst_uri_handler_debug);
 #define GST_CAT_DEFAULT gst_uri_handler_debug
 
+enum
+{
+  NEW_URI,
+  LAST_SIGNAL
+};
+
+static guint gst_uri_handler_signals[LAST_SIGNAL] = { 0 };
+
 static void gst_uri_handler_base_init (gpointer g_class);
 
 GType
 gst_uri_handler_get_type (void)
 {
-  static GType urihandler_type = 0;
+  static volatile gsize urihandler_type = 0;
 
-  if (G_UNLIKELY (urihandler_type == 0)) {
+  if (g_once_init_enter (&urihandler_type)) {
+    GType _type;
     static const GTypeInfo urihandler_info = {
       sizeof (GstURIHandlerInterface),
       gst_uri_handler_base_init,
@@ -72,14 +81,16 @@ gst_uri_handler_get_type (void)
       NULL
     };
 
-    urihandler_type = g_type_register_static (G_TYPE_INTERFACE,
+    _type = g_type_register_static (G_TYPE_INTERFACE,
         "GstURIHandler", &urihandler_info, 0);
 
     GST_DEBUG_CATEGORY_INIT (gst_uri_handler_debug, "GST_URI", GST_DEBUG_BOLD,
         "handling of URIs");
+    g_once_init_leave (&urihandler_type, _type);
   }
   return urihandler_type;
 }
+
 static void
 gst_uri_handler_base_init (gpointer g_class)
 {
@@ -90,12 +101,13 @@ gst_uri_handler_base_init (gpointer g_class)
     /**
      * GstURIHandler::new-uri:
      * @handler: The #GstURIHandler which emitted the signal
-     * @uri: The new URI, or NULL if the URI was removed
+     * @uri: (transfer none): The new URI, or NULL if the URI was removed
      *
      * The URI of the given @handler has changed.
      */
 
-    g_signal_new ("new-uri", GST_TYPE_URI_HANDLER, G_SIGNAL_RUN_LAST,
+    gst_uri_handler_signals[NEW_URI] =
+        g_signal_new ("new-uri", GST_TYPE_URI_HANDLER, G_SIGNAL_RUN_LAST,
         G_STRUCT_OFFSET (GstURIHandlerInterface, new_uri), NULL, NULL,
         gst_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
     initialized = TRUE;
@@ -196,8 +208,7 @@ escape_string_internal (const gchar * string, UnsafeCharacterSet mask)
   return result;
 }
 
-/**
- * escape_string:
+/* escape_string:
  * @string: string to be escaped
  *
  * Escapes @string, replacing any and all special characters
@@ -231,7 +242,7 @@ unescape_character (const char *scanner)
     return -1;
   }
 
-  second_digit = hex_to_int (*scanner++);
+  second_digit = hex_to_int (*scanner);
   if (second_digit < 0) {
     return -1;
   }
@@ -239,8 +250,7 @@ unescape_character (const char *scanner)
   return (first_digit << 4) | second_digit;
 }
 
-/**
- * unescape_string:
+/* unescape_string:
  * @escaped_string: an escaped URI, path, or other string
  * @illegal_characters: a string containing a sequence of characters
  * considered "illegal", '\0' is automatically in this list.
@@ -336,7 +346,7 @@ gst_uri_protocol_is_valid (const gchar * protocol)
  * @uri: A URI string
  *
  * Tests if the given string is a valid URI identifier. URIs start with a valid
- * protocol followed by "://" and maybe a string identifying the location.
+ * scheme followed by ":" and maybe a string identifying the location.
  *
  * Returns: TRUE if the string is a valid URI
  */
@@ -349,7 +359,7 @@ gst_uri_is_valid (const gchar * uri)
 
   gst_uri_protocol_check_internal (uri, &endptr);
 
-  return (*endptr == ':' && *(endptr + 1) == '/' && *(endptr + 2) == '/');
+  return *endptr == ':';
 }
 
 /**
@@ -369,14 +379,14 @@ gst_uri_get_protocol (const gchar * uri)
   g_return_val_if_fail (uri != NULL, NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
-  colon = strstr (uri, "://");
+  colon = strstr (uri, ":");
 
   return g_ascii_strdown (uri, colon - uri);
 }
 
 /**
  * gst_uri_has_protocol:
- * @uri: an URI string
+ * @uri: a URI string
  * @protocol: a protocol string (e.g. "http")
  *
  * Checks if the protocol of a given valid URI matches @protocol.
@@ -394,7 +404,7 @@ gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
   g_return_val_if_fail (protocol != NULL, FALSE);
   g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
 
-  colon = strstr (uri, "://");
+  colon = strstr (uri, ":");
 
   if (colon == NULL)
     return FALSE;
@@ -411,8 +421,11 @@ gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
  * the hostname if one is specified. The returned string must be freed using
  * g_free().
  *
- * Returns: The location for this URI. Returns NULL if the URI isn't valid. If
- * the URI does not contain a location, an empty string is returned.
+ * Free-function: g_free
+ *
+ * Returns: (transfer full) (array zero-terminated=1): the location for this
+ *     URI. Returns NULL if the URI isn't valid. If the URI does not contain
+ *     a location, an empty string is returned.
  */
 gchar *
 gst_uri_get_location (const gchar * uri)
@@ -424,6 +437,8 @@ gst_uri_get_location (const gchar * uri)
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
   colon = strstr (uri, "://");
+  if (!colon)
+    return NULL;
 
   unescaped = unescape_string (colon + 3, "/");
 
@@ -442,19 +457,22 @@ gst_uri_get_location (const gchar * uri)
 #endif
 
   GST_LOG ("extracted location '%s' from URI '%s'", GST_STR_NULL (unescaped),
-      uri);;
+      uri);
   return unescaped;
 }
 
 /**
  * gst_uri_construct:
  * @protocol: Protocol for URI
- * @location: Location for URI
+ * @location: (array zero-terminated=1) (transfer none): Location for URI
  *
  * Constructs a URI for a given valid protocol and location.
  *
- * Returns: a new string for this URI. Returns NULL if the given URI protocol
- * is not valid, or the given location is NULL.
+ * Free-function: g_free
+ *
+ * Returns: (transfer full) (array zero-terminated=1): a new string for this
+ *     URI. Returns NULL if the given URI protocol is not valid, or the given
+ *     location is NULL.
  */
 gchar *
 gst_uri_construct (const gchar * protocol, const gchar * location)
@@ -490,9 +508,9 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
 
   if (!GST_IS_ELEMENT_FACTORY (feature))
     return FALSE;
-  factory = GST_ELEMENT_FACTORY (feature);
+  factory = GST_ELEMENT_FACTORY_CAST (feature);
 
-  if (gst_element_factory_get_uri_type (factory) != entry->type)
+  if (factory->uri_type != entry->type)
     return FALSE;
 
   protocols = gst_element_factory_get_uri_protocols (factory);
@@ -512,11 +530,8 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
 }
 
 static gint
-sort_by_rank (gconstpointer a, gconstpointer b)
+sort_by_rank (GstPluginFeature * first, GstPluginFeature * second)
 {
-  GstPluginFeature *first = GST_PLUGIN_FEATURE (a);
-  GstPluginFeature *second = GST_PLUGIN_FEATURE (b);
-
   return gst_plugin_feature_get_rank (second) -
       gst_plugin_feature_get_rank (first);
 }
@@ -571,11 +586,11 @@ gst_uri_protocol_is_supported (const GstURIType type, const gchar * protocol)
  * gst_element_make_from_uri:
  * @type: Whether to create a source or a sink
  * @uri: URI to create an element for
- * @elementname: Name of created element, can be NULL.
+ * @elementname: (allow-none): Name of created element, can be NULL.
  *
  * Creates an element for handling the given URI.
  *
- * Returns: a new element or NULL if none could be created
+ * Returns: (transfer full): a new element or NULL if none could be created
  */
 GstElement *
 gst_element_make_from_uri (const GstURIType type, const gchar * uri,
@@ -598,10 +613,11 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     return NULL;
   }
 
-  possibilities = g_list_sort (possibilities, sort_by_rank);
+  possibilities = g_list_sort (possibilities, (GCompareFunc) sort_by_rank);
   walk = possibilities;
   while (walk) {
-    if ((ret = gst_element_factory_create (GST_ELEMENT_FACTORY (walk->data),
+    if ((ret =
+            gst_element_factory_create (GST_ELEMENT_FACTORY_CAST (walk->data),
                 elementname)) != NULL) {
       GstURIHandler *handler = GST_URI_HANDLER (ret);
 
@@ -638,8 +654,13 @@ gst_uri_handler_get_uri_type (GstURIHandler * handler)
 
   iface = GST_URI_HANDLER_GET_INTERFACE (handler);
   g_return_val_if_fail (iface != NULL, GST_URI_UNKNOWN);
-  g_return_val_if_fail (iface->get_type != NULL, GST_URI_UNKNOWN);
-  ret = iface->get_type ();
+  g_return_val_if_fail (iface->get_type != NULL
+      || iface->get_type_full != NULL, GST_URI_UNKNOWN);
+
+  if (iface->get_type != NULL)
+    ret = iface->get_type ();
+  else
+    ret = iface->get_type_full (G_OBJECT_TYPE (handler));
   g_return_val_if_fail (GST_URI_TYPE_IS_VALID (ret), GST_URI_UNKNOWN);
 
   return ret;
@@ -652,9 +673,9 @@ gst_uri_handler_get_uri_type (GstURIHandler * handler)
  * Gets the list of protocols supported by @handler. This list may not be
  * modified.
  *
- * Returns: the supported protocols.
- * Returns NULL if the @handler isn't implemented properly, or the @handler
- * doesn't support any protocols.
+ * Returns: (transfer none) (array zero-terminated=1) (element-type utf8): the
+ *     supported protocols. Returns NULL if the @handler isn't implemented
+ *     properly, or the @handler doesn't support any protocols.
  */
 gchar **
 gst_uri_handler_get_protocols (GstURIHandler * handler)
@@ -685,9 +706,9 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
  *
  * Gets the currently handled URI.
  *
- * Returns: the URI currently handled by the @handler.
- * Returns NULL if there are no URI currently handled. The returned
- * string must not be modified or freed.
+ * Returns: (transfer none): the URI currently handled by the @handler.
+ *   Returns NULL if there are no URI currently handled. The
+ *   returned string must not be modified or freed.
  */
 G_CONST_RETURN gchar *
 gst_uri_handler_get_uri (GstURIHandler * handler)
@@ -721,7 +742,7 @@ gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
 {
   GstURIHandlerInterface *iface;
   gboolean ret;
-  gchar *new_uri, *protocol, *location;
+  gchar *new_uri, *protocol, *location, *colon;
 
   g_return_val_if_fail (GST_IS_URI_HANDLER (handler), FALSE);
   g_return_val_if_fail (gst_uri_is_valid (uri), FALSE);
@@ -731,8 +752,11 @@ gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
   g_return_val_if_fail (iface->set_uri != NULL, FALSE);
 
   protocol = gst_uri_get_protocol (uri);
-  location = gst_uri_get_location (uri);
-  new_uri = g_strdup_printf ("%s://%s", protocol, location);
+
+  colon = strstr (uri, ":");
+  location = g_strdup (colon);
+
+  new_uri = g_strdup_printf ("%s%s", protocol, location);
 
   ret = iface->set_uri (handler, uri);
 
@@ -756,5 +780,5 @@ gst_uri_handler_new_uri (GstURIHandler * handler, const gchar * uri)
 {
   g_return_if_fail (GST_IS_URI_HANDLER (handler));
 
-  g_signal_emit_by_name (handler, "new-uri", uri);
+  g_signal_emit (handler, gst_uri_handler_signals[NEW_URI], 0, uri);
 }
