@@ -37,13 +37,13 @@ G_BEGIN_DECLS
 /**
  * GST_BASE_TRANSFORM_SINK_NAME:
  *
- * the name of the templates for the sink pad
+ * The name of the templates for the sink pad.
  */
 #define GST_BASE_TRANSFORM_SINK_NAME	"sink"
 /**
  * GST_BASE_TRANSFORM_SRC_NAME:
  *
- * the name of the templates for the source pad
+ * The name of the templates for the source pad.
  */
 #define GST_BASE_TRANSFORM_SRC_NAME	"src"
 
@@ -103,7 +103,6 @@ typedef struct _GstBaseTransformPrivate GstBaseTransformPrivate;
 
 /**
  * GstBaseTransform:
- * @element: the parent element.
  *
  * The opaque #GstBaseTransform data structure.
  */
@@ -144,12 +143,12 @@ struct _GstBaseTransform {
 
 /**
  * GstBaseTransformClass:
- * @transform_caps: Optional.  given the pad in this direction and the given
+ * @transform_caps: Optional.  Given the pad in this direction and the given
  *                  caps, what caps are allowed on the other pad in this
  *                  element ?
  * @fixate_caps:    Optional. Given the pad in this direction and the given
  *                  caps, fixate the caps on the other pad.
- * @transform_size: Optional. given the size of a buffer in the given direction
+ * @transform_size: Optional. Given the size of a buffer in the given direction
  *                  with the given caps, calculate the size in bytes of a buffer
  *                  on the other pad with the given other caps.
  *                  The default implementation uses get_unit_size and keeps
@@ -182,6 +181,14 @@ struct _GstBaseTransform {
  *                         analysis can return a subbuffer or even just
  *                         increment the reference to the input buffer (if in
  *                         passthrough mode)
+ * @before_transform: Optional. Since 0.10.22
+ *                    This method is called right before the base class will
+ *                    start processing. Dynamic properties or other delayed
+ *                    configuration could be performed in this method.
+ * @accept_caps: Optional. Since 0.10.30
+ *               Subclasses can override this method to check if @caps can be
+ *               handled by the element. The default implementation might not be
+ *               the most optimal way to check this in all cases.
  *
  * Subclasses can override any of the available virtual methods or not, as
  * needed. At minimum either @transform or @transform_ip need to be overridden.
@@ -231,8 +238,13 @@ struct _GstBaseTransformClass {
   /* src event */
   gboolean      (*src_event)      (GstBaseTransform *trans, GstEvent *event);
 
+  void          (*before_transform)  (GstBaseTransform *trans, GstBuffer *buffer);
+
+  gboolean      (*accept_caps)  (GstBaseTransform *trans, GstPadDirection direction,
+                                         GstCaps *caps);
+
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE - 1];
+  gpointer       _gst_reserved[GST_PADDING_LARGE - 3];
 };
 
 GType           gst_base_transform_get_type         (void);
@@ -253,6 +265,12 @@ void		gst_base_transform_set_qos_enabled  (GstBaseTransform *trans,
 		                                     gboolean enabled);
 gboolean	gst_base_transform_is_qos_enabled   (GstBaseTransform *trans);
 
+void            gst_base_transform_set_gap_aware    (GstBaseTransform *trans,
+                                                     gboolean gap_aware);
+
+void		gst_base_transform_suggest          (GstBaseTransform *trans,
+	                                             GstCaps *caps, guint size);
+void		gst_base_transform_reconfigure      (GstBaseTransform *trans);
 G_END_DECLS
 
 #endif /* __GST_BASE_TRANSFORM_H__ */
